#ifndef __SCHED_HH__
#define __SCHED_HH__

#include <vector>
#include "pattern.hh"
#include "tinst.hh"

/***
 * A NOP pattern in the schedule slot corresponds to a NOP
 * instruciton. All slots are filled with NOPs if the current
 * instruction set is ejected.
 */
class Schedule {

public:	
  Schedule(int _omega);
  ~Schedule();
  Schedule(const Schedule &);
  //return true if insertion successful, and returns offset, that is
  //additional slots that needed to be inserted. If _offset is greater
  //than 0, then the returned offset should be 0.
  std::pair< bool, int > insert(TargetInstruction *, 
				std::map< char *, std::pair< char *, lcc::Node >, ltstr > &,
				pattern_t *, int _offset = 0);
  int size(void) { return s.size(); }
  void advance(int _offset);
  bool available_slot(void);
  friend std::ostream & operator<<(std::ostream &, const Schedule &);

public:
  //map: str->str: op->instance
  typedef std::pair< TargetInstruction *, 
		     std::map< char *, std::pair< char *, lcc::Node>, ltstr> > inst_instance_t;
  typedef std::pair< pattern_t *, inst_instance_t > slot_t;
  std::vector< std::vector< slot_t > > s;
  int omega;
};

std::ostream & operator<<(std::ostream &, const Schedule &);

#endif //__SCHED_HH__
